
function spacegame:enter()

--Initialization
  bulletTimer = 0
  graphicsTimer = 0

  --General GFX
  gfx = {}

  gfx.bg = love.graphics.newImage("gfx/bg.png")
  gfx.boom = love.graphics.newImage("gfx/boom.png")
  gfx.gameover = love.graphics.newImage("gfx/gameover.png")

  gfx.poke1 = love.graphics.newImage("gfx/enemies/pokeatron/1.png")
  gfx.poke2 = love.graphics.newImage("gfx/enemies/pokeatron/2.png")
  gfx.pokeBullet = love.graphics.newImage("gfx/enemies/pokeatron/hand.png")

  gfx.powerups = {}
  gfx.powerups.firerate = love.graphics.newImage("gfx/items/firerate.png")

  background = gfx.bg
  poke = gfx.poke1
  enemyAnimTimer = Timer.new()

  --Sound and music
  Music = {}
  Music.level1 = love.audio.newSource("sfx/Music/level1.mp3")


  sfx = {}
  sfx.playerDeath = love.audio.newSource("sfx/playerDeath.wav", "static")
  sfx.playerDeath:setVolume(0.8)
  love.audio.play(Music.level1)

  --Classes

  --Player class
  ship = class:new()
  ship.hp = 100
  ship.X = 100
  ship.Y = 100
  ship.middleX = 160
  ship.middleY = 22
  ship.width = 120
  ship.height = 43
  ship.defaultSpeed = 400
  ship.focusSpeed = 200
  ship.speed = ship.defaultSpeed
  ship.anim = {}
  ship.anim[1] = love.graphics.newImage("gfx/ship1.png")
  ship.anim[2] = love.graphics.newImage("gfx/ship2.png")
  ship.anim[3] = love.graphics.newImage("gfx/ship3.png")
  ship.graphic = ship.anim[1]

  --Bullet code
  ship.bullets = {}

  --Gun code
  laser = class:new()
  laser.delay = 0.5
  laser.damage = 1
  laser.graphic = love.graphics.newImage("gfx/guns/laser.png")

  beam = class:new()
  beam.delay = 11
  beam.damage = 9001

  gun = laser:new()

  --Make a new player
  player = ship:new()
  player.isAlive = true
  player.lives = 3

  --Enemies
  enemies = {}
  enemies.amount = 0
  enemies.timer = Timer.new()

  enemies.bullets = {}

  reloadingenemies = {}

  powerups = {}

  --Enemy timer for spawn and animation
    enemies.timer:addPeriodic(3, function() spawnPoke() end)
    enemyAnimTimer:addPeriodic(0.5, function() cyclePoke() end)

end


--Draw stuff here!
function spacegame:draw()

  --Background
  love.graphics.draw(background, 0, 0)

  --Powerups
  for i, v in ipairs(powerups) do
    love.graphics.draw(v.graphic, v.X, v.Y)
  end

  --Bullets and Enemies
  for i, v in ipairs(enemies.bullets) do
    love.graphics.draw(gfx.pokeBullet, v.X, v.Y, 0, 0.75, 0.75)
  end
  for i, v in ipairs(player.bullets) do
    love.graphics.draw(gun.graphic, v.X, v.Y, 0, 1.5, 1.5)
  end
  for i, v in ipairs(enemies) do
  love.graphics.draw(poke, v.X, v.Y, 0, 0.75, 0.75)
  end

  --Player, is at bottom so he's always on top (Draw logic)
  love.graphics.draw(player.graphic, player.X, player.Y)
end


--Update
function spacegame:update(dt)

--Various timers, remember that delta time is in seconds
Timer.update(dt)
enemies.timer:update(dt)
enemyAnimTimer:update(dt)
graphicsTimer = graphicsTimer + (1 * dt)
bulletTimer = bulletTimer + (1 * dt)


--Cycle through graphics
if graphicsTimer < 0.25 then
  if player.isAlive == true then
    player.graphic = player.anim[1]
  end
  elseif graphicsTimer < 0.5 and graphicsTimer >= 0.25 then
  if player.isAlive == true then
    player.graphic = player.anim[2]
  end
  elseif graphicsTimer < 0.75 and graphicsTimer >= 0.5 then
  if player.isAlive == true then
    player.graphic = player.anim[3]
  end
end

if graphicsTimer > 0.75 then
  graphicsTimer = 0
end

--Remove bullets and enemies
local remEnemy = {}
local remBullet = {}
local remEnemyBullet = {}
local remPowerup = {}

--Position stuff on the player
gun.X = player.X + 100
gun.Y = player.Y + 18

player.middleX = player.X + player.width / 2
player.middleY = player.Y + player.height / 2

--Check if player is dead
if player.hp <= 0 and player.isAlive == true then
  playerDie()
end

--Move bullets
  for i,v in ipairs(player.bullets) do
    v.X = v.X + v.speed * dt
    if v.X > (Width + 100) then
      table.insert(remBullet, i)
    end

        --Check for collision with enemies
        for ii,vv in ipairs(enemies) do
            if CheckCollision(v.X,v.Y,v.width,v.height,vv.X,vv.Y,vv.width,vv.height) then
                -- mark that enemy for removal
                vv.HP = vv.HP - gun.damage
                if vv.HP < 1 then
                  table.insert(remEnemy, ii)
                  enemies.amount = enemies.amount - 1
                  if math.random(1,10) == 10 then
                    spawnPowerup("firerate", vv.X, vv.Y)
                  end
                end
                -- mark the shot to be removed
                table.insert(remBullet, i)
            end
        end
  end

--Check powerup pickup
for i,v in ipairs(powerups) do
  if CheckCollision(player.X, player.Y, player.width, player.height, v.X, v.Y, v.width, v.height) then
  	table.insert(remPowerup, i)
    gun.delay = gun.delay / 1.25
  end
end

for i,v in ipairs(enemies.bullets) do
  v.X = v.X - v.speed * dt
  if CheckCollision(player.X, player.Y, player.width, player.height, v.X, v.Y, v.width, v.height) then
    table.insert(remEnemyBullet, i)
	player.hp = player.hp - 50
  end
end


--Enemy AI
for i,v in ipairs(enemies) do
if v.isReloading == false then
  dx = player.middleX - v.X
  dy = player.middleY - v.Y
  distance = math.sqrt(dx*dx+dy*dy)
  v.X = v.X + (dx / distance * v.speed * dt)
  v.Y = v.Y + (dy / distance * v.speed * dt)
  v.middleX = v.X + (v.width / 2)
  v.middleY = v.Y + (v.height / 2)
  if distanceFrom(player.middleX, player.middleY, v.X, v.Y) < 300 and v.Y > player.Y and v.Y < (player.Y + player.height) and v.isReloading == false then
    shootPoke(v.middleX, (v.middleY + 7))
	v.isReloading = true
	v.reloadtimer = 0
  end
else --Reload behavior
  v.X = v.X + v.reloadspeed * dt
  v.reloadtimer = v.reloadtimer + dt
  if v.reloadtimer >= v.reloadtime then
    v.isReloading = false
    v.reloadtimer = 0
  end
  if v.X > (Width - v.width) then
    v.X = (Width - v.width)
  end
end
  if CheckCollision(v.X,v.Y,v.width,v.height,player.X,player.Y,player.width,player.height) and player.isAlive == true then
    playerDie()
  end
end

--Remove enemies and bullets
for i,v in ipairs(remEnemy) do
  table.remove(enemies, v)
end

for i,v in ipairs(remBullet) do
  table.remove(player.bullets, v)
end

for i,v in ipairs(remEnemyBullet) do
  table.remove(enemies.bullets, v)
end
for i,v in ipairs(remPowerup) do
  table.remove(powerups, v)
end

--Move enemies and check collision with player
--[[
  for i,v in ipairs(enemies) do
      v.X = v.X - (20 * dt)
      dx = (player.X - v.X) * (v.speed * dt)
      dy = (player.Y - v.Y) * (v.speed * dt)
      v.X = v.X + (dx * dt)
      v.Y = v.Y + (dy * dt)
    if CheckCollision(v.X,v.Y,v.width,v.height,player.X,player.Y,player.width,player.height) and player.isAlive == true then
    playerDie()
    end
  end
]]

--Run control code

--Movement handler

	if player.isAlive == true then

	 local xSpeed = 0
	 local ySpeed = 0

	 if love.keyboard.isDown("down") then
	 ySpeed = ySpeed + 1
	 end
	 if love.keyboard.isDown("up") then
	 ySpeed = ySpeed - 1
	 end
	 if love.keyboard.isDown("left") then
	 xSpeed = xSpeed - 1
	 end
	 if love.keyboard.isDown("right") then
	 xSpeed = xSpeed + 1
	 end
	 if xSpeed ~= 0 and ySpeed ~= 0 then
	 xSpeed = (xSpeed * 1.41)/2
	 ySpeed = (ySpeed * 1.41)/2
	 end

	 player.X = player.X + (xSpeed * player.speed) * dt
	 player.Y = player.Y + (ySpeed * player.speed) * dt

		--Continuous shots
	if love.keyboard.isDown("z") then
	  shoot()
	end

  end

--Don't let the player out of the screen
if player.X < 0 then
  player.X = 0
end
if player.X > Width - player.width then
  player.X = Width - player.width
end
if player.Y < 0 then
  player.Y = 0
end
if player.Y > Height - player.height then
  player.Y = Height - player.height
end

end

function spacegame:keypressed(key)
    if key == "lshift" then
      player.speed = player.focusSpeed
    end
    if key == "escape" then
      love.event.push("quit")
    end
    if key == "p" then
      playerDie()
    end
    if key == "x" then
    bomb()
    end
    if key == "rctrl" then
      debug.debug()
    end
	if key == "o" then
	  player.lives = 1
	  player.hp = 1
	  playerDie()
	end
end

function spacegame:keyreleased(key)
  if key == "lshift" then
  player.speed = player.defaultSpeed
  end
end


function spacegame:leave()
  for _, v in pairs(Music) do
    love.audio.stop(v)
  end
end

--Non-LOVE functions

-- Collision detection function.
-- Checks if a and b overlap.
-- w and h mean width and height.
function CheckCollision(ax1,ay1,aw,ah, bx1,by1,bw,bh)
  local ax2,ay2,bx2,by2 = ax1 + aw, ay1 + ah, bx1 + bw, by1 + bh
  return ax1 < bx2 and ax2 > bx1 and ay1 < by2 and ay2 > by1
end

function spawnPoke()
  if enemies.amount < 8 then
    enemies.poke = {}
    enemies.poke.X = 1300
    enemies.poke.Y = math.random(1,719)
    enemies.poke.width = 60
    enemies.poke.height = 60
	enemies.poke.middleX = enemies.poke.X + (enemies.poke.width / 2)
	enemies.poke.middleY = enemies.poke.Y + (enemies.poke.height / 2)
    enemies.poke.speed = 300
	enemies.poke.reloadspeed = 150
	enemies.poke.reloadtime = 2
	enemies.poke.reloadtimer = 0
    enemies.poke.HP = 1
    enemies.poke.isReloading = false
    table.insert(enemies, enemies.poke)
    enemies.amount = enemies.amount + 1
  end
end

function cyclePoke()
  if poke == gfx.poke1 then
    poke = gfx.poke2
  else
    poke = gfx.poke1
  end
end

function shootPoke(pokeX, pokeY)
  pokeBullet = {}
  pokeBullet.X = pokeX
  pokeBullet.Y = pokeY
  pokeBullet.width = 19
  pokeBullet.height = 17
  pokeBullet.graphic = gfx.pokeBullet
  pokeBullet.speed = 250
  table.insert(enemies.bullets, pokeBullet)
end

function spawnPowerup(type, X, Y)
  if type == "firerate" then
    powerup = {}
    powerup.type = type
    powerup.X = X
    powerup.Y = Y
	powerup.width = 45
	powerup.height = 45
    powerup.graphic = gfx.powerups.firerate
    table.insert(powerups, powerup)
  end
end

function textBox(text)

end

function distanceFrom(x1,y1,x2,y2)
  return math.sqrt((x2 - x1) ^ 2 + (y2 - y1) ^ 2)
end
